#include "ext.h"
#include "z_dsp.h" 

void *sigvmax_class;

typedef struct _sigvmax
{
    t_pxobject	x_obj;
	
    double 		s_rate;
    long		sigvs;
	
    void 		*l_out;					// double outlet for peak
										// void 		*l_out2;				// sigvmax flag (1=limit, 0 = no limit)
	
} t_sigvmax, *t_sigvmax_ptr;

t_int *sigvmax_perform(t_int *w);
void *sigvmax_new(double hold_time_ms,double seuil);
void sigvmax_dsp(t_sigvmax *x, t_signal **sp, short *count);
void sigvmax_assist(t_sigvmax *x, void *b, long m, long a, char *s);
void sigvmax_free(t_sigvmax *x);
void sigvmax_debug(t_sigvmax *x);

void main(void)
{
    setup((t_messlist **) &sigvmax_class, (method) sigvmax_new, (method)sigvmax_free, (short)sizeof(t_sigvmax), 0L,A_DEFFLOAT,A_DEFFLOAT,0);
    addmess((method)sigvmax_dsp, "dsp", A_CANT, 0);
	addmess((method)sigvmax_assist,"assist",A_CANT,0);
	addmess((method)sigvmax_debug,"debug",0);
    post("gn.sigvmax~ v0.2 UB compiled %s %s  Gilbert Nouno @ 4dec2006",__DATE__,__TIME__);
	dsp_initclass(); // set object class to work with MSP
	rescopy('STR#',4100);
}

void sigvmax_assist(t_sigvmax *x, void *b, long m, long a, char *s)
{
	assist_string(4100,m,a,1,2,s);
}

void *sigvmax_new(double hold_time_ms,double seuil)
{
    t_sigvmax *x = (t_sigvmax *)newobject(sigvmax_class);
    dsp_setup((t_pxobject *)x,1); // sigvmax has one input signal
								  // x->l_out 	= outlet_new((t_pxobject *)x, 0L); // one control outlet
    outlet_new((t_pxobject *)x, "signal");
	
    x->x_obj.z_misc	=	Z_NO_INPLACE; // in & out independants    
    
    x->s_rate		=	44100;			// peut-on demander le sample rate ici ???????????????
    return (x);
}

void sigvmax_free(t_sigvmax *x) {
	dsp_free((t_pxobject *)x);
}

void sigvmax_dsp(t_sigvmax *x, t_signal **sp, short *count)
{
	x->s_rate	=	sys_getsr();
	x->sigvs	=	sys_getblksize(); // ne marche pas pourquoi ???
	dsp_add(sigvmax_perform, 4, x, sp[0]->s_vec, sp[1]->s_vec, sp[0]->s_n);
}

t_int *sigvmax_perform(t_int *w)	{
	float		*in, *out, *in_copy, *out_copy;
	double		vector_peak;
	
	long		n,n_copy;
	
	t_sigvmax *x=	(t_sigvmax *)(w[1])	;
	in 			= 	(float *)(w[2])		;		// vecteur d'entre
	out 		= 	(float *)(w[3])		;		// vecteur de sortie
	n 			= 	w[4]				;		// sigvs
	
	vector_peak		=	0				;
	
	in_copy		=	in					;
	out_copy	=	out					;
	n_copy		=	n					;
	
	while (n--) {
		if ( *in > vector_peak) vector_peak=*in;
		*in++;
	}
	
	n = n_copy;
	
	while (n--) {
		*out++ = vector_peak;
	}
		
	return (w+5);
}

void sigvmax_debug(t_sigvmax *x)	{
	post("s_rate =%f", x->s_rate);
	post("sigvs  =%f", x->sigvs);
}
